from gtk import gdk
import gedit

class Plugin(gedit.Plugin):
    def activate(self, window):
        self.views = {}

    def deactivate(self, window):
        for view, signal in self.views.items():
            view.disconnect(signal)

    def update_ui(self, window):
        buffer = window.get_active_document()
        view = window.get_active_view()

        if buffer == None or view == None:
            return

        language = buffer.get_language()
        if language == None:
            return

        if language.get_name() == 'Ruby' and view not in self.views:
            self.views[view] = (view.connect('key-press-event',
                                self.on_key_press, buffer))

    def on_key_press(self, view, event, buffer):
        key_name = gdk.keyval_name(event.keyval)
        if key_name not in ('Return', 'BackSpace') or \
           len(buffer.get_selection_bounds()) != 0:
            return

        if view.get_insert_spaces_instead_of_tabs():
            self.indent = ' ' * view.props.tab_width
        else:
            self.indent = '\t'

        if key_name == 'Return':
            line = self._get_current_line(buffer)
            stline = line.split('|')[0].strip()
            if (stline.startswith('def')
            	or stline.startswith('class')
            	or stline.startswith('begin')
            	or stline.startswith('module')
            	or stline.startswith('if')
            	or stline.startswith('unless')
            	or stline.startswith('for')
            	or stline.startswith('while')
            	or stline.startswith('until')
            	or stline.startswith('case')
            	or stline.endswith('{')
            	or stline.endswith('[')
            	or stline.endswith('/')
            	or stline.endswith('do')):
                old_indent = line[:len(line) - len(line.lstrip())]
                indent = '\n' + old_indent + self.indent
                buffer.insert_interactive_at_cursor(indent, True)
                self._scroll_to_cursor(buffer, view)
                return True

            elif (stline.endswith('}')
            	or stline.endswith(']')
            	or stline.endswith('/')
            	or stline.endswith('end')):
            	old_indent = line[:len(line) - len(line.lstrip())]
            	next = old_indent[:len(old_indent)-len(self.indent)]
            	words = next + line.lstrip() + '\n' + next
                self._delete_before_cursor(buffer, len(line))
                buffer.insert_interactive_at_cursor(words, True)
                self._scroll_to_cursor(buffer, view)
                return True #Return a 'True' to disable a default '\n'

            elif (stline.startswith('rescue')
            	or stline.startswith('when')
            	or stline.startswith('else')
            	or stline.startswith('elsif')):
            	old_indent = line[:len(line) - len(line.lstrip())]
            	next = old_indent[:len(old_indent)-len(self.indent)]
            	words = next + line.lstrip() + '\n' + old_indent
                self._delete_before_cursor(buffer, len(line))
                buffer.insert_interactive_at_cursor(words, True)
                self._scroll_to_cursor(buffer, view)
                return True

        if key_name == 'BackSpace':
            line = self._get_current_line(buffer)

            if line.strip() == '' and line != '':
                length = len(self.indent)
                nb_to_delete = len(line) % length or length
                self._delete_before_cursor(buffer, nb_to_delete)
                self._scroll_to_cursor(buffer, view)
                return True

    def _delete_before_cursor(self, buffer, nb_to_delete):
        cursor_position = buffer.get_property('cursor-position')
        iter_cursor = buffer.get_iter_at_offset(cursor_position)
        iter_before = buffer.get_iter_at_offset(cursor_position - nb_to_delete)
        buffer.delete_interactive(iter_before, iter_cursor, True)

    def _get_current_line(self, buffer):
        iter_cursor = self._get_iter_cursor(buffer)
        iter_line = buffer.get_iter_at_line(iter_cursor.get_line())
        return buffer.get_text(iter_line, iter_cursor)

    def _get_current_line_nb(self, buffer):
        iter_cursor = self._get_iter_cursor(buffer)
        return iter_cursor.get_line()

    def _get_iter_cursor(self, buffer):
        cursor_position = buffer.get_property('cursor-position')
        return buffer.get_iter_at_offset(cursor_position)

    def _scroll_to_cursor(self, buffer, view):
        lineno = self._get_current_line_nb(buffer) + 1
        insert = buffer.get_insert()
        view.scroll_mark_onscreen(insert)

